@@ -23,6 +23,11 @@
 # TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+## @mainpage
+# The @e ldapom python module provides a simple LDAP object mapper.
+# @author Florian Richter
+#
+
 import ldap
 
 LDAPOM_VERBOSE = False
@@ -91,7 +96,8 @@ class LdapConnection(object):
     ## @param login The dn we are authenticating with
     ## @param password The password for the login dn
     ## @param certfile If using SSL/TLS this is certificate of the server
-    def __init__(self, uri, base, login, password, certfile=None):
+    ## @param timelimit Set the timelimit a search request may take
+    def __init__(self, uri, base, login, password, certfile=None, timelimit=30):
         """
         Create a new LdapConnection.
 
@@ -114,6 +120,8 @@ class LdapConnection(object):
         self._connect()
         ## Defines how long we will wait for result answers from the LDAP server
         self._timeout = 0
+        ## Set timelimit, python-ldap defaults to 30
+        ldap.set_option(ldap.OPT_TIMELIMIT, timelimit)
 
     ## @return None
     def _connect(self):
@@ -328,11 +336,13 @@ class LdapConnection(object):
 
 class LdapAttribute(object):
     """
-        Holds an set of LDAP-Attributes with the same name.
-        All changes are recorded, so they can be push to ldap_modify
-        directly
+        Holds a set of LDAP-Attributes with the same name.
+        
+        All changes are recorded, so they can be pushed to ldap_modify directly.
     """
 
+    ## Creates a new attribute with the given @p name and @p value. If @p add is @e False (default), the current value
+    # is overwritten, else appended.
     def __init__(self, name, value, add=False):
         self._name = unicode(name)
         self._replace_all = False
@@ -350,57 +360,92 @@ class LdapAttribute(object):
             else:
                 self._values = [unicode(value)]
 
+    ## @return Integer
     def __len__(self):
+        """
+        The number of values set for this attribute
+        """
         return len(self._values)
 
+    ## @return String
     def __unicode__(self):
-        "if there's only one item, return it directly"
+        """
+        unicode value of this attribute
+        """
+        # if there's only one item, return it directly
         if len(self._values) == 1:
-            return self._values[0]
-        return unicode(self._values)
+            return unicode(self._values[0], "utf-8")
+        return unicode(self._values, "utf-8")
 
+    ## @returns the String representation of the object.
     def __repr__(self):
+        """
+        literal representation
+        """
         return "<LdapAttribute: %s=%s>" % (self._name, self.__unicode__())
 
+    ## @param value String the additional value
+    ## @return None
     def append(self, value):
-        "add an attribute"
+        """
+        add an attribute
+        """
         if not value in self._values:
             self._values.append(unicode(value))
             self._changes.append((ldap.MOD_ADD, self._name, unicode(value)))
 
+    ## @param value String the to-be-removed value
+    ## @return None
     def remove(self, value):
-        "remove an attribute"
-        if unicode(value) in self._values:
+        """
+        remove an attribute
+        """
+        if str(value) in self._values:
             self._values.remove(unicode(value))
             self._changes.append((ldap.MOD_DELETE, self._name, unicode(value)))
 
+    ## Membership test operator (<em>in</em> and <em>not in</em>), tests if the attribute contains the @p item.
+    ## @return Boolean
     def __contains__(self, item):
         return self._values.__contains__(item)
 
+    ## @returns the item identified by @p key
     def __getitem__(self, key):
         return self._values[key]
 
+    ## Sets the value of @p key to @p value.
+    # @return None
     def __setitem__(self, key, value):
         self._replace_all = True
         self._values[key] = unicode(value)
 
+    ## Deletes the value identified by its @p key
+    # @return None
     def __delitem__(self, key):
         self._replace_all = True
         self._values.remove(key)
 
+    ## @return Iterator
     def __iter__(self):
         return self._values.__iter__()
 
+    ## Sets single @p value, discards all existing ones.
+    ## @return None
     def set_value(self, value):
-        "set single value, discard all existing ones"
+        """
+        Sets single value, discards all existing ones
+        """
         if type(value) == list:
             self._values = [unicode(x) for x in value]
         else:
             self._values = [unicode(value)]
         self._replace_all = True
 
+    ## @return Array
     def get_change_list(self):
-        "get all changes to this attribute in ldap_modify-syntax"
+        """
+        get all changes to this attribute in ldap_modify-syntax
+        """
         if self._replace_all:
             if len(self) == 0:
                 return [(ldap.MOD_DELETE, self._name, None)]
@@ -409,21 +454,30 @@ class LdapAttribute(object):
             return change_list
         return self._changes
 
+    ## @return None
     def discard_change_list(self):
-        "called when attribute-changes were successfully saved"
+        """
+        called when attribute-changes were successfully saved
+        """
         self._changes = []
         self._replace_all = False
 
 
 class LdapNode(object):
     """
-        Holds an ldap-object represented by the dn (distinguishable name).
-        attributes are fetched from ldapserver lazily, so you can create objects
-        without network traffic.
+    Holds an ldap-object represented by the dn (distinguishable name).
+    attributes are fetched from ldapserver lazily, so you can create objects
+    without network traffic.
     """
 
+    ## Creates the node using the given connection and dn.
+    # @param conn The connection string
+    # @param dn The DN to use
+    # @param new default: @e False
     def __init__(self, conn, dn, new=False):
-        "Create lazy Node Object from dn"
+        """
+        Create lazy Node Object from dn
+        """
         self._conn = conn
         self._dn = unicode(dn)
         self._valid = True
@@ -434,6 +488,7 @@ class LdapNode(object):
         else:
             self._attr = None
 
+    ## @return None
     def retrieve_attributes(self):
         """Retrieves the node's attributes from the database.
 
@@ -443,25 +498,29 @@ class LdapNode(object):
         _dn, attributes_dict = list(self._conn.query(base=self._dn, scope=ldap.SCOPE_BASE))[0]
         self._load_attributes(attributes_dict)
 
+    ## @return None
     def _load_attributes(self, attributes_dict):
         self._attr = dict([
             (attr_name, LdapAttribute(attr_name, attr_values))
                 for attr_name, attr_values in attributes_dict.items()
         ])
 
+    ## @returns the value of the attribute identified by its @p name. Attributes starting with <em>is_*</em> are mapped
+    # to a check, if the objectClass is present.
     def __getattr__(self, name):
         """
             get an ldap-attribute
-            * attributes starting with is_* are mapped to a check, if the objectClass is present
         """
         if self._attr == None:
             self.retrieve_attributes()
         if name.startswith("is_"):
-            return name[3:] in self._attr[u"objectClass"]
+            return name[3:] in self._attr[u'objectClass']
         if name in self._attr:
             return self._attr[name]
         raise AttributeError('Cannot find attribute %s' % name)
 
+    ## Sets the @p value of the attribute identified by its @p name.
+    # @return None
     def __setattr__(self, name, value):
         "set ldap attribute"
         # handle private attributes the default way
@@ -474,30 +533,39 @@ class LdapNode(object):
         else:
             self._attr[name] = LdapAttribute(name, value, add=True)
 
+    ## Deletes the attribute identified by its @p name.
+    # @return None
     def __delattr__(self, name):
         if self._attr == None:
             self.retrieve_attributes()
         del self._attr[name]
         self._to_delete.append(name)
 
+    ## @returns the unicode DN of the node.
     def __unicode__(self):
         return self._dn
 
+    ## @returns the string DN of the node.
     def __str__(self):
         return self._dn.encode("utf-8")
 
+    ## @returns the String representation of the object.
     def __repr__(self):
         return "<LdapNode: %s>" % self._dn
 
+    ## Saves any changes made to the object.
+    ## @return None
     def save(self):
-        """Save any changes to the object"""
+        """
+        Save any changes to the object
+        """
         if self._attr == None:
             # No changes yet
             return
         if self._new:
             change_list = [ (_encode_utf8(x._name), [_encode_utf8(y) for y in x]) for x in self._attr.values() ]
             if LDAPOM_VERBOSE:  # pragma: no cover
-                print "ldap_add: %s" % change_list
+                print("ldap_add: {0}".format(change_list))
             self._conn.add(_encode_utf8(self._dn), change_list)
         else:
             change_list = [ (ldap.MOD_DELETE, _encode_utf8(x), None) for x in self._to_delete ]
@@ -506,13 +574,14 @@ class LdapNode(object):
             if len(change_list) == 0:
                 return
             if LDAPOM_VERBOSE:  # pragma: no cover
-                print "ldap_modify: %s" % change_list
+                print("ldap_modify: {0}".format(change_list))
             self._conn.modify(_encode_utf8(self._dn), change_list)
         self._new = False
         self._to_delete = []
         for attr in self._attr.values():
             attr.discard_change_list()
 
+    ## @return None
     def delete(self):
         """
         delete this object in ldap
@@ -527,6 +596,8 @@ class LdapNode(object):
         self._conn.delete(_encode_utf8(self._dn))
         self._valid = False
 
+    ## @return Boolean
+    ## @param password String Password which will be used for authentication
     def check_password(self, password):
         """
         check password for this ldap-object
@@ -538,6 +609,8 @@ class LdapNode(object):
         """
         return self._conn.authenticate( _encode_utf8(self._dn), _encode_utf8(password) )
 
+    ## @return None
+    ## @param password String new password (plain text as hashes are done by the LDAP server)
     def set_password(self, password):
         """
         set password for this ldap-object immediately
